@@ -1,5 +1,5 @@
 /*  Black-box: guess where the crystals are !                                           
-    Copyright (C) 2000 Karl Bartel                                              
+    Copyright (C) 2000-2008 Karl Bartel                                              
                                                                                 
     This program is free software; you can redistribute it and/or modify        
     it under the terms of the GNU General Public License as published by        
@@ -28,10 +28,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <SDL.h>
+#include <SDL_image.h>
 #include <SDL_timer.h>
+#include <SDL_mixer.h>
 #include <signal.h>
 #include "SFont.h"
-#include "mixer/SDL_mixer.h"
 
        int fullscreen=1,difficulty=4,trihigh=1,sound=1;   //settings
 
@@ -48,9 +49,9 @@
        char text[80];
        char xline[9];
        char yline[11];
-       char think[11][9];
+       char think[12][9];
        char light[11][9];
-       char real[11][9];
+       char real[12][9];
 
 #define PATHNUM 5
 char DATAPATH[200]=DATADIR;
@@ -120,11 +121,11 @@ SDL_Surface *LoadImage(char *datafile, int transparent)   // reads one GIF into
   int i=0;
 
   sprintf(filename,"%s/gfx/%s",DATAPATH,datafile);
-  pic=IMG_Load(filename);
+  pic = IMG_Load(filename);
   while ( pic == NULL ) {
     strcpy(DATAPATH,PATH[i]);
     sprintf(filename,"%s/gfx/%s",DATAPATH,datafile);
-    pic=IMG_Load(filename);
+    pic = IMG_Load(filename);
     i++;
     
     if (i>PATHNUM)
@@ -235,9 +236,9 @@ void init_blit()
   blit(695,480,help_pic);
   
   sprintf(text,"Level %d",level_num);
-  rect.x=800-TextWidth(text);
+  rect.x=800-111;
   rect.y=300;
-  rect.w=TextWidth(text);
+  rect.w=111;
   rect.h=Font->h;
   SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format,0,0,0));
   PutString(screen,800-TextWidth(text),300,text);
@@ -252,7 +253,7 @@ void ReadCommandLine(char *argv[])
     if ((strcmp(argv[i],"--nofullscreen")==0)||(strcmp(argv[i],"-f")==0)) {fullscreen=0;} else
     if ((strcmp(argv[i],"--nosound")==0)||(strcmp(argv[i],"-s")==0)) {sound=0;} else
     if (((strcmp(argv[i],"--levelnum")==0)&&(argv[i+1]))||((strcmp(argv[i],"-l")==0)&&(argv[i+1])))
-      {i++;level_num=atoi(argv[i]);printf("starting with level #%d\n",level_num);LevelGiven=1;} else
+      {i++;level_num=atoi(argv[i])%100000;printf("starting with level #%d\n",level_num);LevelGiven=1;} else
     {
 	if (!((strcmp(argv[i],"-h")==0)||(strcmp(argv[i],"--help")==0))) printf("Unknown parameter: \"%s\"\n", argv[i]);
 	puts("\nCommand line options:\n");
@@ -394,44 +395,43 @@ void select_shot(int x,int y,int dir,int erase)
   }
 }
 
-output calc_real(int x,int y,int dir)
+output calc_out(char map[12][9],int x,int y,int dir)
 {
   int end=0;
   output out;
   
   while (!end)
   {
-    if (dir==0) 
+    switch (dir)
     {
+    case 0:
       if (y==0) {end=1;} else
-      if (real[x-1][y-1]==1) {dir=1;} else
-      if (real[x][y-1]==1) {dir=3;}
+      if (map[x-1][y-1]==1) {dir=1;} else
+      if (map[x][y-1]==1) {dir=3;}
       else {y--;}
-    }
-    if (dir==1)
-    {
+    break;
+    case 1:
       if (x==12) {end=1;} else
-      if (real[x][y]==1) {dir=0;} else
-      if (real[x][y-1]==1) {dir=2;}
+      if (map[x][y]==1) {dir=0;} else
+      if (map[x][y-1]==1) {dir=2;}
       else {x++;}
-    }
-    if (dir==2)
-    {
+    break;
+    case 2:
       if (y==9) {end=1;} else
-      if (real[x][y]==1) {dir=3;} else
-      if (real[x-1][y]==1) {dir=1;}
+      if (map[x][y]==1) {dir=3;} else
+      if (map[x-1][y]==1) {dir=1;}
       else {y++;}
-    }
-    if (dir==3)
-    {
+    break;
+    case 3:
       if (x==0) {end=1;} else
-      if (real[x-1][y]==1) {dir=0;} else
-      if (real[x-1][y-1]==1) {dir=2;}
+      if (map[x-1][y]==1) {dir=0;} else
+      if (map[x-1][y-1]==1) {dir=2;}
       else {x--;}
+    break;
     }
-    if ((x>30)||(y>20)||(y<0)||(y<0))
+    if ((x>12)||(y>9)||(x<0)||(y<0))
     {
-      printf("ERROR: x or y out of range in function hidden, quitting now.\n");
+      fprintf(stderr,"ERROR: calc_out: x or y out of range, quitting now.\n");
       free_memory();
       exit(1);
     }
@@ -443,55 +443,6 @@ output calc_real(int x,int y,int dir)
   return(out);
 }
 
-output calc_think(int x,int y,int dir)
-{
-  int end=0;
-  output out;
-  
-  while (!end)
-  {
-    if (dir==0) 
-    {
-      if (y==0) {end=1;} else
-      if (think[x-1][y-1]==1) {dir=1;} else
-      if (think[x][y-1]==1) {dir=3;}
-      else {y--;}
-    }
-    if (dir==1)
-    {
-      if (x==12) {end=1;} else
-      if (think[x][y]==1) {dir=0;} else
-      if (think[x][y-1]==1) {dir=2;}
-      else {x++;}
-    }
-    if (dir==2)
-    {
-      if (y==9) {end=1;} else
-      if (think[x][y]==1) {dir=3;} else
-      if (think[x-1][y]==1) {dir=1;}
-      else {y++;}
-    }
-    if (dir==3)
-    {
-      if (x==0) {end=1;} else
-      if (think[x-1][y]==1) {dir=0;} else
-      if (think[x-1][y-1]==1) {dir=2;}
-      else {x--;}
-    }
-    if ((x>30)||(y>20)||(y<0)||(y<0))
-    {
-      printf("ERROR: x or y out of range in function hidden, quitting now.\n");
-      free_memory();
-      exit(1);
-    }
-  }
-  out.x=x;
-  out.y=y;
-  out.dir=dir;
-  
-  return(out);
-}
-
 void blit_screen()
 {
   int x,y;
@@ -523,23 +474,23 @@ void blit_screen()
     }
   }
   for (x=0;x<=10;x++){
-    if ((calc_real(x,0,2).x==calc_think(x,0,2).x)
-       &&(calc_real(x,0,2).y==calc_think(x,0,2).y)
-       &&(calc_real(x,0,2).dir==calc_think(x,0,2).dir))
+    if ((calc_out(real,x,0,2).x==calc_out(think,x,0,2).x)
+       &&(calc_out(real,x,0,2).y==calc_out(think,x,0,2).y)
+       &&(calc_out(real,x,0,2).dir==calc_out(think,x,0,2).dir))
     {blit(x*50+75,0,edge1_pic);}
     else {blit(x*50+75,0,edge1red_pic);}
   }
   for (y=0;y<=8;y++){
-    if ((calc_real(0,y,1).x==calc_think(0,y,1).x)
-       &&(calc_real(0,y,1).y==calc_think(0,y,1).y)
-       &&(calc_real(0,y,1).dir==calc_think(0,y,1).dir))
+    if ((calc_out(real,0,y,1).x==calc_out(think,0,y,1).x)
+       &&(calc_out(real,0,y,1).y==calc_out(think,0,y,1).y)
+       &&(calc_out(real,0,y,1).dir==calc_out(think,0,y,1).dir))
     {blit(0,y*50+75,edge2_pic);}
     else {blit(0,y*50+75,edge2red_pic);}
   }  
   for (x=0;x<=10;x++){
-    if ((calc_real(x,9,0).x==calc_think(x,9,0).x)
-       &&(calc_real(x,9,0).y==calc_think(x,9,0).y)
-       &&(calc_real(x,9,0).dir==calc_think(x,9,0).dir))
+    if ((calc_out(real,x,9,0).x==calc_out(think,x,9,0).x)
+       &&(calc_out(real,x,9,0).y==calc_out(think,x,9,0).y)
+       &&(calc_out(real,x,9,0).dir==calc_out(think,x,9,0).dir))
     {blit(x*50+75,562,edge3_pic);}
     else {blit(x*50+75,562,edge3red_pic);}
   }
@@ -564,63 +515,21 @@ void blit_screen()
 
 void hidden(int x,int y,int dir)  //0=up 1=right 2=down 3=left
 {
-  int end=0,x_in,y_in,dir_in;
-
-  x_in=x;
-  y_in=y;  
-  dir_in=dir;
-  
   select_shot(x,y,dir,0);
-  while (!end)
+  output out=calc_out(real,x,y,dir);
+
+  if ((out.x==x)&&(out.y==y)&&((dir==out.dir+2)||(dir==out.dir-2)))
   {
-//    printf("x:%d  y:%d  dir=%d\n",x,y,dir);
-    if (dir==0) 
-    {
-      if (y==0) {end=1;} else
-      if (real[x-1][y-1]==1) {dir=1;} else
-      if (real[x][y-1]==1) {dir=3;}
-      else {y--;}
-    }
-    if (dir==1)
-    {
-      if (x==12) {end=1;} else
-      if (real[x][y]==1) {dir=0;} else
-      if (real[x][y-1]==1) {dir=2;}
-      else {x++;}
-    }
-    if (dir==2)
-    {
-      if (y==9) {end=1;} else
-      if (real[x][y]==1) {dir=3;} else
-      if (real[x-1][y]==1) {dir=1;}
-      else {y++;}
-    }
-    if (dir==3)
-    {
-      if (x==0) {end=1;} else
-      if (real[x-1][y]==1) {dir=0;} else
-      if (real[x-1][y-1]==1) {dir=2;}
-      else {x--;}
-    }
-    if ((x>30)||(y>20)||(y<0)||(y<0))
-    {
-      printf("ERROR: x or y out of range in function hidden, quitting now.\n");
-      free_memory();
-      exit(1);
-    }
-  }
-  if ((x==x_in)&&(y==y_in)&&((dir_in==dir+2)||(dir_in==dir-2)))
-  {
-    select_shot(x,y,dir_in,1);
-    select_shot(x,y,dir,0);
-    select_shot(x,y,dir,1);    
+    select_shot(x,y,dir,1);
+    select_shot(x,y,out.dir,0);
+    select_shot(x,y,out.dir,1);    
   }
   else
   {
-    select_shot(x,y,dir,0);
+    select_shot(out.x,out.y,out.dir,0);
     SDL_Delay(500);
-    select_shot(x_in,y_in,dir_in,1);
     select_shot(x,y,dir,1);
+    select_shot(out.x,out.y,out.dir,1);
   }
 }
 
@@ -646,7 +555,7 @@ void show_real()
 
 void generate_field()
 {
-  int x,y,crystal_num;
+  int x,y;
   
   for (x=0;x<=10;x++){
     for (y=0;y<=8;y++){
@@ -654,9 +563,9 @@ void generate_field()
       light[x][y]=0;
     }
   }
-  while ((crystal_num>difficulty*3)||(crystal_num<(difficulty-1)*3))
+  for (;;)
   {
-    crystal_num=0;
+    int crystal_num=0;
     srand(level_num);
     for (x=0;x<=9;x++){
       for (y=0;y<=7;y++){
@@ -669,14 +578,17 @@ void generate_field()
         if (real[x][y]==1) {crystal_num++;}
       }
     }  
-    sprintf(text,"Black-Box: Level #%d",level_num);
     if (LevelGiven) {
 	LevelGiven=0;
 	break;
     }
-    level_num=(((unsigned)time(NULL)+SDL_GetTicks())%100000);
+    if (((difficulty-1)*3<=crystal_num)&&(crystal_num<=difficulty*3)) {
+        break;
+    }
+    level_num=(level_num+1)%100000;
 //    printf("%d\n",crystal_num); 
   }
+  sprintf(text,"Black-Box: Level #%d",level_num);
   for (x=0;x<=10;x++)
   {
     yline[x]=0;
@@ -801,21 +713,21 @@ int IsItComplete()
   int x,y,won=1;
   for (x=0;x<=11;x++)
   {
-    if (calc_real(x,0,2).x!=calc_think(x,0,2).x) {won=0;}
-    if (calc_real(x,0,2).y!=calc_think(x,0,2).y) {won=0;}
-    if (calc_real(x,0,2).dir!=calc_think(x,0,2).dir) {won=0;}
+    if (calc_out(real,x,0,2).x!=calc_out(think,x,0,2).x) {won=0;}
+    if (calc_out(real,x,0,2).y!=calc_out(think,x,0,2).y) {won=0;}
+    if (calc_out(real,x,0,2).dir!=calc_out(think,x,0,2).dir) {won=0;}
   }
   for (x=0;x<=11;x++)
   {
-    if (calc_real(x,9,0).x!=calc_think(x,9,0).x) {won=0;}
-    if (calc_real(x,9,0).y!=calc_think(x,9,0).y) {won=0;}
-    if (calc_real(x,9,0).dir!=calc_think(x,9,0).dir) {won=0;}
+    if (calc_out(real,x,9,0).x!=calc_out(think,x,9,0).x) {won=0;}
+    if (calc_out(real,x,9,0).y!=calc_out(think,x,9,0).y) {won=0;}
+    if (calc_out(real,x,9,0).dir!=calc_out(think,x,9,0).dir) {won=0;}
   }
   for (y=0;y<=9;y++)
   {
-    if (calc_real(0,y,1).x!=calc_think(0,y,1).x) {won=0;}
-    if (calc_real(0,y,1).y!=calc_think(0,y,1).y) {won=0;}
-    if (calc_real(0,y,1).dir!=calc_think(0,y,1).dir) {won=0;}
+    if (calc_out(real,0,y,1).x!=calc_out(think,0,y,1).x) {won=0;}
+    if (calc_out(real,0,y,1).y!=calc_out(think,0,y,1).y) {won=0;}
+    if (calc_out(real,0,y,1).dir!=calc_out(think,0,y,1).dir) {won=0;}
   }
   return(won);  
 }
@@ -985,7 +897,7 @@ void init_sound()
 
 main(int argc, char *argv[])
 {
-  printf("\nBlack-Box version %s, Copyright (C) 2000 Karl Bartel\n",VERSION);
+  printf("\nBlack-Box version %s, Copyright (C) 2000-2008 Karl Bartel\n",VERSION);
   printf("Black-Box comes with ABSOLUTELY NO WARRANTY; for details see COPYING'.\n");
   printf("This is free software, and you are welcome to redistribute it\n");
   printf("under certain conditions.\n");
